/*
 * Copyright 2021 - All rights reserved.
 * Rodrigo Tapia-McClung
 *
 * January 2021
 */

/* globals updateBarChart, barChartData, am4charts, am4core, am4themes_animated, yearList */
/* export makeRadialChart, makeLineChart */

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
function grijalva_theme(target) {
    if (target instanceof am4core.ColorSet) {
        target.list = [
            am4core.color("#00c5ff"),
            am4core.color("#ff5500"),
            am4core.color("#98e600"),
        ];
    }
}

function customizeGrip(grip) {
    // Remove default grip image
    grip.icon.disabled = true;

    // Disable background
    grip.background.disabled = true;

    // Add rotated rectangle as bi-di arrow
    let img = grip.createChild(am4core.Rectangle);
    img.width = 7;
    img.height = 7;
    img.fill = am4core.color("#999");
    img.rotation = 45;
    img.align = "center";
    img.valign = "middle";
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ RADAR- LINES

let mainTextColor = getComputedStyle(document.body).getPropertyValue('--main-text-color');

const createRadialSeries = (chart, value, category, name, color) => {
    let radialSeries = chart.series.push(new am4charts.RadarSeries());
    radialSeries.dataFields.valueY = value;
    radialSeries.dataFields.categoryX = category;
    radialSeries.strokeWidth = 1.5;
    radialSeries.tooltipText = "{valueY}";
    radialSeries.name = name;
    //radialSeries.connect = false;
    //radialSeries.connectEnds = false;
    radialSeries.stroke = am4core.color(color); // red
    radialSeries.tooltip.getFillFromObject = false;
    radialSeries.tooltipText = "[bold] {name}: [/]{valueY.formatNumber('#,###.0')} Ha[/]";
    radialSeries.tooltip.background.fill = am4core.color(color);
}

const makeRadialRange = (axis, start, end, color, span) => {
    let radialRange = axis.axisRanges.create();
    radialRange.category = start;
    radialRange.endcategory = end;
    radialRange.axisFill.fill = am4core.color(color);
    radialRange.axisFill.fillOpacity = 0.4;
    radialRange.grid.strokeOpacity = 0;
    radialRange.locations.endCategory = span ? span : "";
}

const createLineAxisAndSeries = (chart, field, name, opposite, bullet) => {
    var lineValueAxis = chart.yAxes.push(new am4charts.ValueAxis());
    lineValueAxis.fontSize = 8;

    if (chart.yAxes.indexOf(lineValueAxis) != 0) {
        lineValueAxis.syncWithAxis = chart.yAxes.getIndex(0);
    }

    var lineSeries = chart.series.push(new am4charts.LineSeries());
    lineSeries.dataFields.valueY = field;
    lineSeries.dataFields.dateX = "date";
    lineSeries.strokeWidth = 2;
    lineSeries.yAxis = lineValueAxis;
    lineSeries.name = name;
    //series.tooltipText = "{name}: [bold]{valueY.formatNumber('#,###.0')} Ha[/]";
    lineSeries.tooltipText = "[bold]{name}: [/]{valueY.formatNumber('#,###.0')} Ha[/]";
    lineSeries.tooltipText.fontSize = 8;
    lineSeries.tensionX = 0.8;
    lineSeries.showOnInit = true;

    lineValueAxis.renderer.line.strokeOpacity = 1;
    lineValueAxis.renderer.line.strokeWidth = 2;
    lineValueAxis.renderer.line.stroke = lineSeries.stroke;
    lineValueAxis.renderer.labels.template.fill = lineSeries.stroke;
    lineValueAxis.renderer.opposite = opposite;
}

const makeLineRange = (axis, start, width, color) => {
    yearList.forEach( y => {
        var range = axis.axisRanges.create();
        range.date = new Date(y, start, 1, 0, 0, 0);
        range.endDate = new Date(y, start+width, 1, 0, 0, 0);
        range.axisFill.fill = am4core.color(color);
        range.axisFill.fillOpacity = 0.4;
    })
}
const makeRadialChart = (data, months, years) => {
    let radialChart = am4core.create("radial_chart", am4charts.RadarChart);

    let radialTitle = radialChart.titles.create();
    radialTitle.text = "Superficie total por año";
    radialTitle.fontSize = 10;
    radialTitle.marginBottom = 10;
    radialTitle.fill = am4core.color(mainTextColor);

    radialChart.data = data;

    /* Create axes */
    let radialCategoryAxis = radialChart.xAxes.push(new am4charts.CategoryAxis());
    radialCategoryAxis.dataFields.category = "month";
    radialCategoryAxis.fontSize = 9;
    radialCategoryAxis.renderer.labels.template.fill = am4core.color(mainTextColor);
    radialCategoryAxis.renderer.grid.template.stroke = am4core.color(mainTextColor);

    let radialValueAxis = radialChart.yAxes.push(new am4charts.ValueAxis());
    radialValueAxis.renderer.grid.template.stroke = am4core.color(mainTextColor);
    radialValueAxis.extraMin = 0.2;
    radialValueAxis.extraMax = 0.2;
    radialValueAxis.tooltip.disabled = true;
    radialValueAxis.fontSize = 9;
    radialValueAxis.renderer.labels.template.fill = am4core.color(mainTextColor);
    radialValueAxis.renderer.grid.template.stroke = am4core.color(mainTextColor);
    //radialValueAxis.renderer.labels.template.fontSize = 15;

    /* Create and configure series */
    let seriesColor = ["#F98293", "#2AC2BB", "#FDA259"];
    years.forEach( (year, i) => {
        createRadialSeries(radialChart, year, "month", year, seriesColor[i]);
    });

    radialChart.scrollbarX = new am4core.Scrollbar();
    radialChart.scrollbarX.minHeight = 1;
    radialChart.scrollbarX.parent = radialChart.bottomAxesContainer;
    radialChart.scrollbarY = new am4core.Scrollbar();
    radialChart.scrollbarY.minWidth = 1;
    customizeGrip(radialChart.scrollbarX.startGrip);
    customizeGrip(radialChart.scrollbarX.endGrip);
    customizeGrip(radialChart.scrollbarY.startGrip);
    customizeGrip(radialChart.scrollbarY.endGrip);

    let radialRangeColors = ["#E1DBC8", "#595952", "#838278", "#B1AE9F"];
    makeRadialRange(radialCategoryAxis, "enero", "marzo", radialRangeColors[0], 2);
    makeRadialRange(radialCategoryAxis, "marzo", "mayo", radialRangeColors[1], 2);
    makeRadialRange(radialCategoryAxis, "mayo", "septiembre", radialRangeColors[2], 4);
    makeRadialRange(radialCategoryAxis,     "septiembre", "diciembre", radialRangeColors[3], 3);
    makeRadialRange(radialCategoryAxis, "diciembre", "diciembre", radialRangeColors[0], 1);

    radialChart.cursor = new am4charts.RadarCursor();

    radialChart.legend = new am4charts.Legend();
    //radialChart.legend.fontSize = 10;
    radialChart.legend.fontSize = 9;
    radialChart.legend.labels.template.fill = am4core.color(mainTextColor);

    let legenddata = [{ name: "Estiaje", fill: am4core.color("#595952") },
        { name: "Tropical", fill: am4core.color("#838278") },
        { name: "Tropical-Invernal", fill: am4core.color("#B1AE9F") },
        { name: "Invernal", fill: am4core.color("#E1DBC8") }
    ];

    radialChart.legend.data = legenddata;
    radialChart.legend.fontSize = 10;
    radialChart.legend.labels.template.fill = am4core.color(mainTextColor);
    let markerTemplate = radialChart.legend.markers.template;
    markerTemplate.width = 20;
    markerTemplate.height = 20;
}

const makeLineChart = (data) => {

    let lineChart = am4core.create("linesChart", am4charts.XYChart);
    let lineTitle = lineChart.titles.create();
    lineTitle.text = "Superficie por tipo de cuerpo";
    lineTitle.fontSize = 10;
    lineTitle.marginBottom = 15;
    lineTitle.fill = am4core.color(mainTextColor);

    lineChart.data = data;

    var lineDateAxis = lineChart.xAxes.push(new am4charts.DateAxis());
    lineDateAxis.renderer.minGridDistance = 50;
    lineDateAxis.fontSize = 8;
    lineDateAxis.renderer.labels.template.fill = am4core.color(mainTextColor);

    // Create series
    createLineAxisAndSeries(lineChart, "permanente", "Agua permanente", false, "circle");
    createLineAxisAndSeries(lineChart, "temporal", "Áreas temporalmente inundadas", true, "triangle");
    createLineAxisAndSeries(lineChart, "vegetacion", "Suelos húmedos-vegetación acuática", true, "rectangle");

    // create line ranges
    makeLineRange(lineDateAxis, 0, 2, "#E1DBC8");
    makeLineRange(lineDateAxis, 2, 2, "#595952");
    makeLineRange(lineDateAxis, 4, 4, "#838278");
    makeLineRange(lineDateAxis, 8, 3, "#B1AE9F");
    makeLineRange(lineDateAxis, 11, 1, "#E1DBC8");

    // Add cursor
    lineChart.cursor = new am4charts.XYCursor();
    lineChart.scrollbarX = new am4core.Scrollbar();
    lineChart.scrollbarX.minHeight = 1;
    lineChart.scrollbarX.parent = lineChart.bottomAxesContainer;
    customizeGrip(lineChart.scrollbarX.startGrip);
    customizeGrip(lineChart.scrollbarX.endGrip);
    // Style scrollbar
}

am4core.ready(function() {

    // Themes begin
    am4core.useTheme(am4themes_animated);
    am4core.useTheme(grijalva_theme);
    //let mainTextColor = getComputedStyle(document.body).getPropertyValue('--main-text-color');
    // Themes end

    // area chart
    let areaChart = am4core.create("stackedAreaChart", am4charts.XYChart);

    let areaTitle = areaChart.titles.create();
    areaTitle.text = "Superficie acumulada";
    areaTitle.fontSize = 10;
    areaTitle.marginBottom = 15;
    areaTitle.fill = am4core.color(mainTextColor);

    // chart.dateFormatter.inputDateFormat = "dd-mm-yyyy";
    var areaDateAxis = areaChart.xAxes.push(new am4charts.DateAxis());
    areaDateAxis.renderer.minGridDistance = 60;
    areaDateAxis.fontSize = 8;
    areaDateAxis.renderer.labels.template.fill = am4core.color(mainTextColor);
    areaDateAxis.startLocation = 0.5;
    areaDateAxis.endLocation = 0.5;
    areaDateAxis.baseInterval = {
        timeUnit: "month",
        count: 1
    }

    var areaValueAxis = areaChart.yAxes.push(new am4charts.ValueAxis());
    areaValueAxis.tooltip.disabled = true;
    areaValueAxis.fontSize = 8;
    areaValueAxis.renderer.labels.template.fill = am4core.color(mainTextColor);

    //TODO: make these series in a more efficient way
    var areasSeries = areaChart.series.push(new am4charts.LineSeries());
    areasSeries.dataFields.dateX = "date";
    areasSeries.name = "Agua permanente";
    areasSeries.dataFields.valueY = "permanente";
    //areasSeries.tooltipHTML = "<img src='https://www.amcharts.com/lib/3/images/car.png' style='vertical-align:bottom; margin-right: 10px; width:28px; height:21px;'><span style='font-size:14px; color:#000000;'><b>{valueY.value}</b></span>";
    //areasSeries.tooltipText = "[#000]{valueY.value}[/]";
    areasSeries.tooltipText = "[bold]{name}: [/]{valueY.formatNumber('#,###.#')} Ha";
    //areasSeries.tooltip.background.fill = am4core.color("#FFF");
    areasSeries.tooltip.getStrokeFromObject = true;
    areasSeries.tooltip.background.strokeWidth = 3;
    //areasSeries.tooltip.getFillFromObject = false;
    areasSeries.fillOpacity = 0.6;
    areasSeries.strokeWidth = 2;
    areasSeries.stacked = true;

    var areasSeries2 = areaChart.series.push(new am4charts.LineSeries());
    areasSeries2.name = "Áreas temporalmente inundadas";
    areasSeries2.dataFields.dateX = "date";
    areasSeries2.dataFields.valueY = "temporal";
    //areasSeries2.tooltipHTML = "<img src='https://www.amcharts.com/lib/3/images/motorcycle.png' style='vertical-align:bottom; margin-right: 10px; width:28px; height:21px;'><span style='font-size:14px; color:#000000;'><b>{valueY.value}</b></span>";
    //areasSeries2.tooltipText = "[#000]{valueY.value}[/]";
    areasSeries2.tooltipText = "[bold]{name}: [/]{valueY.formatNumber('#,###.#')} Ha";
    //areasSeries2.tooltip.background.fill = am4core.color("#FFF");
    //areasSeries2.tooltip.getFillFromObject = false;
    areasSeries2.tooltip.getStrokeFromObject = true;
    areasSeries2.tooltip.background.strokeWidth = 3;
    areasSeries2.sequencedInterpolation = true;
    areasSeries2.fillOpacity = 0.6;
    areasSeries2.stacked = true;
    areasSeries2.strokeWidth = 2;

    var areasSeries3 = areaChart.series.push(new am4charts.LineSeries());
    areasSeries3.name = "Suelos húmedos-vegetación acuática";
    areasSeries3.dataFields.dateX = "date";
    areasSeries3.dataFields.valueY = "vegetacion";
    //areasSeries3.tooltipHTML = "<img src='https://www.amcharts.com/lib/3/images/bicycle.png' style='vertical-align:bottom; margin-right: 10px; width:28px; height:21px;'><span style='font-size:14px; color:#000000;'><b>{valueY.value}</b></span>";
    //areasSeries3.tooltipText = "[#000]{valueY.value}[/]";
    areasSeries3.tooltipText = "[bold]{name}: [/]{valueY.formatNumber('#,###.#')} Ha";
    //areasSeries3.tooltip.background.fill = am4core.color("#FFF");
    //areasSeries3.tooltip.getFillFromObject = false;
    areasSeries3.tooltip.getStrokeFromObject = true;
    areasSeries3.tooltip.background.strokeWidth = 3;
    areasSeries3.sequencedInterpolation = true;
    areasSeries3.fillOpacity = 0.6;
    areasSeries3.defaultState.transitionDuration = 1000;
    areasSeries3.stacked = true;
    areasSeries3.strokeWidth = 2;

    areaChart.cursor = new am4charts.XYCursor();
    areaChart.snapToSeries = areasSeries2;
    areaChart.cursor.xAxis = areaDateAxis;
    areaChart.scrollbarX = new am4core.Scrollbar();
    areaChart.scrollbarX.minHeight = 1;
    areaChart.scrollbarX.parent = areaChart.bottomAxesContainer;
    customizeGrip(areaChart.scrollbarX.startGrip);
    customizeGrip(areaChart.scrollbarX.endGrip);
    //areaChart.scrollbarX.position = "bottom";

    // Add a legend
    // axis ranges
    //TODO: make these ranges in a more efficient way
    var areaRange = areaDateAxis.axisRanges.create();
    areaRange.date = new Date(2016, 0, 1);
    areaRange.endDate = new Date(2016, 2, 1);
    areaRange.axisFill.fill = am4core.color("#E1DBC8");
    areaRange.axisFill.fillOpacity = 0.4;

    var areaRange1 = areaDateAxis.axisRanges.create();
    areaRange1.date = new Date(2016, 2, 1);
    areaRange1.endDate = new Date(2016, 4, 1);
    areaRange1.axisFill.fill = am4core.color("#595952");
    areaRange1.axisFill.fillOpacity = 0.4;

    var areaRange2 = areaDateAxis.axisRanges.create();
    areaRange2.date = new Date(2016, 4, 1);
    areaRange2.endDate = new Date(2016, 8, 1);
    areaRange2.axisFill.fill = am4core.color("#838278");
    areaRange2.axisFill.fillOpacity = 0.4;

    var areaRange3 = areaDateAxis.axisRanges.create();
    areaRange3.date = new Date(2016, 8, 1);
    areaRange3.endDate = new Date(2016, 11, 1);
    areaRange3.axisFill.fill = am4core.color("#B1AE9F");
    areaRange3.axisFill.fillOpacity = 0.4;

    var areaRange4 = areaDateAxis.axisRanges.create();
    areaRange4.date = new Date(2016, 11, 1);
    areaRange4.endDate = new Date(2017, 2, 1);
    areaRange4.axisFill.fill = am4core.color("#E1DBC8");
    areaRange4.axisFill.fillOpacity = 0.4;

    var areaRange5 = areaDateAxis.axisRanges.create();
    areaRange5.date = new Date(2017, 2, 1);
    areaRange5.endDate = new Date(2017, 4, 1);
    areaRange5.axisFill.fill = am4core.color("#595952");
    areaRange5.axisFill.fillOpacity = 0.4;

    var areaRange6 = areaDateAxis.axisRanges.create();
    areaRange6.date = new Date(2017, 4, 1);
    areaRange6.endDate = new Date(2017, 8, 1);
    areaRange6.axisFill.fill = am4core.color("#838278");
    areaRange6.axisFill.fillOpacity = 0.4;

    var areaRange7 = areaDateAxis.axisRanges.create();
    areaRange7.date = new Date(2017, 8, 1);
    areaRange7.endDate = new Date(2017, 11, 1);
    areaRange7.axisFill.fill = am4core.color("#B1AE9F");
    areaRange7.axisFill.fillOpacity = 0.4;

    var areaRange8 = areaDateAxis.axisRanges.create();
    areaRange8.date = new Date(2017, 11, 1);
    areaRange8.endDate = new Date(2018, 2, 1);
    areaRange8.axisFill.fill = am4core.color("#E1DBC8");
    areaRange8.axisFill.fillOpacity = 0.4;

    var areaRange9 = areaDateAxis.axisRanges.create();
    areaRange9.date = new Date(2018, 2, 1);
    areaRange9.endDate = new Date(2018, 4, 1);
    areaRange9.axisFill.fill = am4core.color("#595952");
    areaRange9.axisFill.fillOpacity = 0.4;

    var areaRange10 = areaDateAxis.axisRanges.create();
    areaRange10.date = new Date(2018, 4, 1);
    areaRange10.endDate = new Date(2018, 8, 1);
    areaRange10.axisFill.fill = am4core.color("#838278");
    areaRange10.axisFill.fillOpacity = 0.4;

    var areaRange11 = areaDateAxis.axisRanges.create();
    areaRange11.date = new Date(2018, 8, 1);
    areaRange11.endDate = new Date(2018, 11, 1);
    areaRange11.axisFill.fill = am4core.color("#B1AE9F");
    areaRange11.axisFill.fillOpacity = 0.4;

    var areaRange12 = areaDateAxis.axisRanges.create();
    areaRange12.date = new Date(2018, 11, 1);
    areaRange12.endDate = new Date(2019, 0, 1);
    areaRange12.axisFill.fill = am4core.color("#E1DBC8");
    areaRange12.axisFill.fillOpacity = 0.4;

    /* Set up cursor behavior */
    let previousDate = "";
    areaChart.cursor.events.on("cursorpositionchanged", function(ev) {
        let xAxis = ev.target.chart.xAxes.getIndex(0);
        /*let axis = dateAxis.positionToDate(dateAxis.toAxisPosition(ev.target.xPosition)),
            currentYear = axis.getFullYear().toString(),
            currentMonth = axis.getMonth().toString();
        console.log(currentYear, currentMonth)*/
        let yy = xAxis.positionToDate(xAxis.toAxisPosition(ev.target.xPosition)).getFullYear().toString();
        let mm = xAxis.positionToDate(xAxis.toAxisPosition(ev.target.xPosition)).getMonth().toString();
        let month = 0,
            startDate = 0,
            endDate = 0;
        if (mm < 9) { month = "0" + (parseInt(mm) + 1).toString(); } else {
            month = parseInt(mm) + 1;
            month.toString();
        }
        let filterDate = new Date(`${yy}-${month}-01T00:00:00`);
        // only update chart when dates in cursor change month
        if (filterDate.valueOf() != previousDate.valueOf()) {
            previousDate = filterDate;
            updateBarChart(filterDate, barChartData);
        }
    })

    // bar chart
    var barChart = am4core.create("dimfrcpo-graph", am4charts.XYChart);

    let barTitle = barChart.titles.create();
    barTitle.text = "Conteo por tipo de cuerpo";
    barTitle.fontSize = 10;
    barTitle.marginBottom = 5;
    barTitle.fill = am4core.color(mainTextColor);

    // Create axes
    //var barCategoryAxis = barChart.yAxes.push(new am4charts.CategoryAxis());
    var barCategoryAxis = barChart.xAxes.push(new am4charts.CategoryAxis());
    barCategoryAxis.dataFields.category = "grupo";
    barCategoryAxis.numberFormatter.numberFormat = "#,###.#";
    barCategoryAxis.renderer.inversed = true;
    barCategoryAxis.renderer.grid.template.location = 0;
    barCategoryAxis.renderer.cellStartLocation = 0.1;
    barCategoryAxis.renderer.cellEndLocation = 0.9;
    barCategoryAxis.fontSize = 10;
    barCategoryAxis.renderer.labels.template.fill = am4core.color(mainTextColor);
    barCategoryAxis.title.text = "Superficie (Ha)";
    barCategoryAxis.title.fill = am4core.color(mainTextColor);

    //var barValueAxis = barChart.xAxes.push(new am4charts.ValueAxis());
    var barValueAxis = barChart.yAxes.push(new am4charts.ValueAxis());
    //barValueAxis.renderer.opposite = true;
    //barValueAxis.calculateTotals = true
    barValueAxis.fontSize = 10;
    barValueAxis.extraMax = 0.05;
    barValueAxis.renderer.labels.template.fill = am4core.color(mainTextColor);
    barValueAxis.max = 1250; // TODO: compute max value from data

    // Create series
    function createSeries(field, name) {
        var barSeries = barChart.series.push(new am4charts.ColumnSeries());
        //barSeries.dataFields.valueX = field;
        //barSeries.dataFields.categoryY = "grupo";
        barSeries.dataFields.valueY = field;
        barSeries.dataFields.categoryX = "grupo";
        //barSeries.dataFields.valueYShow = "count";

        //barSeries.dataFields.categoryY = "grupo"; //"year";
        barSeries.dataFields.categoryX = "grupo"; //"year";
        barSeries.name = name;
        barSeries.columns.template.tooltipText = "[underline][bold]Menor a [bold]{categoryX.formatNumber('#,###.#')} Ha[/]: {valueY.formatNumber('#,###.#')} cuerpos[/]";
        barSeries.columns.template.height = am4core.percent(100);
        barSeries.sequencedInterpolation = true;

        var barValueLabel = barSeries.bullets.push(new am4charts.LabelBullet());
        //barValueLabel.label.text = "{valueX}";
        barValueLabel.label.text = "{valueY}";
        barValueLabel.label.horizontalCenter = "middle";
        //barValueLabel.label.verticalCenter = "top";
        //barValueLabel.label.align = "center";
        barValueLabel.label.dx = 3;
        barValueLabel.label.dy = -5;
        barValueLabel.label.hideOversized = false;
        barValueLabel.label.truncate = false;
        barValueLabel.fontSize = 10;
        //barValueLabel.label.fill = am4core.color(mainTextColor);
        barValueLabel.label.fill = am4core.color("#fff");

    }

    createSeries("ap", "Agua permanente");
    createSeries("at", "Áreas temporalmente inundadas");
    createSeries("av", "Suelos húmedos-vegetación acuática");

}); // end am4core.ready()

//--------------------------------------------------------------------------------------------------------------------